<?php

/**
 * @license http://www.gnu.org/licenses/gpl-3.0.html  GNU GPL v3
 * @copyright  2019 Université Grenoble Alpes
 */

namespace mod_labnbook\fetch;

use Firebase\JWT\JWT;

/**
 * Fetch info from the LabNbook API.
 *
 * This class does not depend on the environment (Moodle user, config, role, etc).
 */
class fetcher
{
    const TIMEOUT = 2; // seconds to wait for LabNbook connection

    /**
     * @var string Root URL to the API
     */
    private $baseUrl;

    /**
     * @var string Secret key for signing JWT
     */
    private $secret;

    /**
     * @var environment base of the JWT payload across requests
     */
    private $environment;

    /**
     * @var \Stdclass { code, message }, e.g. { code: 403, message: "not auth" }
     */
    private $lastError;

    public function __construct(string $baseUrl, string $secret, environment $environment)
    {
        if (!$secret) {
            throw new \Exception("The JWT secret key is empty.");
        }
        $this->baseUrl = \rtrim($baseUrl, '/');
        $this->secret = $secret;
        $this->environment = $environment;
    }

    /**
     * @return \Stdclass {code, message}
     */
    public function getLastError(): ?\Stdclass
    {
        return (object) $this->lastError;
    }

    /**
     * Query the LabNbook API.
     *
     * @param string $verb E.g. "GET"
     * @param string $urlPath E.g. "/v1/mission"
     * @param array $payload Default is empty. Else any assoc array.
     * @return mixed
     */
    public function fetch(string $verb, string $urlPath, array $payload = [])
    {
        $curl = \curl_init();
        \curl_setopt_array($curl, [
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_CUSTOMREQUEST => $verb,
            CURLOPT_URL => $this->createUrl($urlPath),
            CURLOPT_HTTPHEADER => [
                'Content-Type: application/json',
                'Accept: application/json',
                'Authorization: Bearer ' . $this->encodeJwtToken($urlPath, $payload),
            ],
            CURLOPT_CONNECTTIMEOUT => self::TIMEOUT,
        ]);
        $response = \curl_exec($curl);
        if (\curl_errno($curl) !== 0) {
            $this->lastError = (object) [
                'code' => \curl_getinfo($curl, CURLINFO_HTTP_CODE),
                'message' => \curl_error($curl),
            ];
        }
        $error = $this->validateResponse($curl, $response);
        if ($error) {
            \curl_close($curl);
            \error_log("Query $verb $urlPath to labnbook API failed. $error");
            throw new \Exception("HTTP request failed.");
        }

        // send decoded response?
        $contentType = \curl_getinfo($curl, CURLINFO_CONTENT_TYPE);
        \curl_close($curl);
        if ($contentType === 'application/json') {
            return \json_decode($response);
        }
        return $response;
    }

    public function createUrl(string $urlPath): string
    {
        return $this->baseUrl . '/' . \ltrim($urlPath, '/');
    }

    /**
     * Returns the JS source code that will fetch the data in a promise.
     *
     * @param string $verb E.g. "GET"
     * @param string $urlPath E.g. "/v1/mission"
     * @param array $payload Default is empty. Else any assoc array.
     * @return string JS source code
     */
    public function getJsFetch(string $verb, string $urlPath, array $payload = [])
    {
        $url = \json_encode($this->createUrl($urlPath));
        $verb = \json_encode($verb);
        $token = $this->encodeJwtToken($urlPath, $payload);
        // no cache, no cookies, no referrer
        return <<<EOJS
fetch($url, {
    method: $verb,
    cache: 'no-cache',
    credentials: 'omit',
    headers: {
        'Content-Type': 'application/json',
        Authorization: 'Bearer $token',
    },
    mode: 'cors',
    referrer: 'no-referrer',
})
EOJS;
    }

    public function encodeJwtToken(string $urlPath, array $payload): string
    {
        $jwtPayload = $this->environment->createPayload();
        if ($payload) {
            $jwtPayload['data'] = $payload;
        }
        $jwtPayload['dest'] = '/' . \trim($urlPath, '/');
        $jwtPayload['iat'] = \time(); // UTC timestamp
        return JWT::encode($jwtPayload, $this->secret, 'HS256');
    }

    /**
     * @param resource $curl
     * @param resource $response
     * @return string if OK "", else error message
     */
    protected function validateResponse($curl, $response)
    {
        if ($response === false) {
            $this->lastError = (object) ['message' => "Network error? " . \curl_error($curl)];
            return "Network error? " . \curl_error($curl);
        }
        $responseCode = \curl_getinfo($curl, CURLINFO_RESPONSE_CODE);
        if ($responseCode != 200) {
            $this->lastError = \json_decode($response); // Do not verify signature?
            if (\json_last_error() !== JSON_ERROR_NONE) {
                $this->lastError = (object) ['message' => $response];
            }
            $this->lastError->code = $responseCode;
            return "HTTP code $responseCode, response: " . $response;
        }
        return "";
    }
}
