<?php

/**
 * @license http://www.gnu.org/licenses/gpl-3.0.html  GNU GPL v3
 * @copyright  2019 Université Grenoble Alpes
 */

namespace mod_labnbook\fetch;

/**
 * Send queries to the LabnBook API as an authenticated LnB user.
 */
class user extends moodlefetcher
{
    const ERROR_TOKEN_NOTVALID = 401;
    const ERROR_LOGIN_UNKNOWN = 403;
    const ERROR_CREATION_FAILED = 500;

    /**
     * Check that the current user has a Labnbook authentication, valid for the next 30 s.
     */
    public static function isAuthenticated()
    {
        global $SESSION;
        return !empty($SESSION->labnbook->auth->token_exp)
            && $SESSION->labnbook->auth->token_exp > \time() + 30;
    }

    /**
     * @return array mission[]
     */
    public function getMissions()
    {
        return $this->fetch('GET', '/v1/mission');
    }

    /**
     * @return object teamconfig
     */
    public function getTeamconfig($teamconfigId)
    {
        return $this->fetch('GET', '/v1/teamconfig/'.$teamconfigId);
    }

    /**
     * @return int
     */
    public function getNumStartedReports($teamconfigId)
    {
        return $this->fetchIgnore('GET', '/v1/teamconfig/'.$teamconfigId.'/countStartedReports', [], 0);
    }

    /**
     * @return int
     */
    public function getNumSubmittedReports($teamconfigId)
    {
        return $this->fetchIgnore('GET', '/v1/teamconfig/'.$teamconfigId.'/countSubmittededReports', [], 0);
    }

    /**
     * Give a teacher a tutor access to these LnB missions and classes, if it does not exist yet.
     *
     * @param array $teamconfigIds int[]
     */
    public function grantMissionsAccess(array $teamconfigIds)
    {
        $this->fetch(
            'POST',
            "/v1/mission/grant",
            [
                'id_team_configs' => $teamconfigIds,
            ]
        );
    }

    /**
     * Use a LnB mission in a Moodle activity.
     *
     * @param int $missionId
     * @param int $courseId
     * @param string $coursename
     * @param int|null $groupId
     * @param array $teamconfig
     * @return object teamconfig
     */
    public function useMission(int $missionId, int $courseId, string $coursename, ?int $groupId = null, array $participants = [], array $teamconfig = [])
    {
        return $this
            ->fetch(
                'POST',
                "/v1/mission/$missionId/use",
                [
                    'courseid' => $courseId,
                    'coursename' => $coursename,
                    'groupid' => $groupId,
                    'participants' => $participants,
                    'team_config' => $teamconfig,
                ]
            )
            ->team_config;
    }

    /**
     * Update the team config of a Moodle-LnB activity.
     *
     * @param int $missionId
     * @param int $courseId
     * @param string $coursename
     * @param int|null $groupId
     * @param array $teamconfig
     * @return object teamconfig
     */
    public function updateMissionUse(int $missionId, int $courseId, string $coursename, ?int $groupId = null, array $teamconfig = [])
    {
        return $this
            ->fetch(
                'POST',
                "/v1/mission/$missionId/update",
                [
                    'courseid' => $courseId,
                    'coursename' => $coursename,
                    'groupid' => $groupId,
                    'team_config' => $teamconfig
                ]
            )
            ->team_config;
    }

    /**
     * Update the participants of a Moodle-LnB activity.
     * @param int $courseId
     * @param int $groupId
     * @param array $participants
     * @return array
     */
    public function updateParticipants(int $courseId, ?int $groupId = null, array $participants =[])
    {
        return $this
            ->fetchIgnore(
                'POST',
                "/v1/classe/updateParticipants",
                [
                    'courseid' => $courseId,
                    'groupid' => $groupId,
                    'participants' => $participants,
                ],
                []
            );
    }



    /**
     * Return the JS code that fetches the missions.
     *
     * E.g. <?= $u->getJsMissions() ?>.then(missions => for (m of missions) {...})
     *
     * @return string JavaScript Promise
     */
    public function getJsMissions()
    {
        return $this->fetcher->getJsFetch('GET', '/v1/mission')
            . '.then(function(r) {
        if (r.ok) {
            return r.json();
        } else {
            console.error(r);
            throw new Error(r);
        }
    }).then(x => x.data)';
    }

    /**
     * Return a JWT token for the redirection page of LnB, pointing to this report page.
     *
     * @param int $teamconfigId
     * @return string JWT
     * @throws \moodle_exception
     */
    public function getReportTokenForRedirect(int $teamconfigId): string
    {
        $payload = [
            'path' => "/report/enter/$teamconfigId",
        ];
        return $this->fetcher->encodeJwtToken('/v1/redirect', $payload);
    }

    /**
     * Return a JWT token for the redirection page of LnB, pointing to the table of reports.
     *
     * @param int $teamconfigId
     * @return string JWT
     * @throws \moodle_exception
     */
    public function getReportsTokenForRedirect(int $teamconfigId): string
    {
        $payload = [
            'path' => "/teacher/reports/?team_config=$teamconfigId",
        ];
        return $this->fetcher->encodeJwtToken('/v1/redirect', $payload);
    }

    /**
     * Return a JWT token for the redirection page of LnB, pointing to mission edition page.
     *
     * @param int $missionid
     * @return string JWT
     * @throws \moodle_exception
     */
    public function getMissionEditTokenForRedirect(int $missionId): string
    {
        $payload = [
            'path' => "/teacher/mission/$missionId",
        ];
        return $this->fetcher->encodeJwtToken('/v1/redirect', $payload);
    }

    /**
     * Return a JWT token for the redirection page of LNB pointing to class edition page
     *
     * @param int $courseid
     * @param int $groupid
     * @return string JWT
     * @throws \moodle_exception
     */
    public function getClassEditTokenForRedirect(int $id_team_config): string
    {
        $payload = [
            'path' => "/teacher/students?id_team_config=".$id_team_config,
        ];
        return $this->fetcher->encodeJwtToken('/v1/redirect', $payload);
    }

    /**
     * Return a JWT token for the redirection page of LnB, pointing to the config of this teaming.
     *
     * @param int $teamconfigId
     * @return string JWT
     * @throws \moodle_exception
     */
    public function getTeamingTokenForRedirect(int $teamconfigId): string
    {
        $payload = [
            'path' => "/teacher/students#teamconfig=$teamconfigId",
        ];
        return $this->fetcher->encodeJwtToken('/v1/redirect', $payload);
    }

    /**
     * Return a JWT token for the redirection page of LnB, point to the page that lists missions.
     *
     * @param int $teamconfigId
     * @return string JWT
     * @throws \moodle_exception
     */
    public function getMissionTokenForRedirect(): string
    {
        $payload = [
            'path' => "/teacher/missions",
        ];
        return $this->fetcher->encodeJwtToken('/v1/redirect', $payload);
    }

    protected function getEnvironment(): environment
    {
        global $SESSION;
        if (empty($SESSION->labnbook->auth->token_exp)) {
            throw new \Exception("User token for the Labnbook API does not exist.");
        }
        $env = parent::getEnvironment();
        $env->orig = "user";
        $env->token_exp = $SESSION->labnbook->auth->token_exp;
        return $env;
    }

    protected function getSigningKey(): string
    {
        global $SESSION;
        if (empty($SESSION->labnbook->auth->token)) {
            throw new \Exception("User token for the Labnbook API does not exist.");
        }
        return $SESSION->labnbook->auth->token;
    }
}
